import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.Vector;
import javax.swing.*;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.StreamTokenizer;
import H.*;

class EnvMap implements Drawable
{
    public Vector obstacles;   // bunch of shapes...
    public Vector observed;
    public Vector wireless;	// holds wireless information
    public AffineTransform AT;
    
    public static final int MAP = 0;
    public static final int IR = 1;
    public static final int BLANK = 2;
    
    // conversion factors for HMC: _X = -576, _Y = -65, SCALE = 2.54
    public static double MAP_OFFSET_X = -576;
    public static double MAP_OFFSET_Y = -65;
    public static double MAP_SCALE = 2.54;
    
    public static double env_min_x, env_min_y, env_max_x, env_max_y, size_x, size_y;
    
    public int type;
    
    // for IR map
    public GeneralPath ir_nw;
    public GeneralPath ir_n;
    public GeneralPath ir_ne;
    public GeneralPath wi;
    
    
    public Line2D.Double extraSeg = null;
    
    // Cubberly auditorium units of paces
    /*
    public double[][] hc = {
    {86,0,0},
    {40,0,9.5},
    {46.5,40,7},
    {49.5,46.5,9},
    {70,68.5,10},
    {70,68.5,11.5},
    {86,84.5,10},
    {86,84.5,11.5}
    };
    
    public double[][] vc = {
    {9.5,7,40},
    {9,7,46.5},
    {30,9,49.5},
    {11.5,10,68.5},
    {11.5,10,70},
    {11.5,10,84.5},
    {30,0,86}
    };
    */
    
    
    // for real map
    public double[][] hc = {  
    {13, 7, 26},
    {338, 13, 24},
    {1004, 338, 20},
    {1079, 1004, 7},
    {1346, 1079, 20},
    {13, 7, 110},
    {195, 13, 114},
    {537, 195, 111},
    {537, 523, 130},
    {537, 523, 169},
    {537, 525, 333},
    {537, 525, 372},
    {609, 537, 502},
    {615, 609, 383},
    {615, 609, 347},
    {615, 609, 300},
    {615, 609, 264},
    {924, 609, 111},
    {960, 924, 118},
    {978, 960, 111},
    {1014, 978, 115},
    {1244, 1014, 111},
    {1346, 1244, 400}
    };
    
    public double[][] vc = {
    {26, 24, 13},
    {26, 110, 7},
    {372, 333, 525},
    {383, 347, 615},
    {24, 20, 338},
    {20, 7, 1004},
    {20, 7, 1079},
    //{290, 20, 1346},
    //{400, 330, 1346},
    {400, 20, 1346},
    {114, 110, 13},
    {114, 111, 195},
    {130, 111, 537},
    {169, 130, 523},
    {333, 169, 537},
    {502, 372, 537},
    {502, 383, 609},
    {347, 300, 609},
    {300, 264, 615},
    {264, 111, 609},
    {118, 111, 924},
    {118, 111, 960},
    {115, 111, 978},
    {115, 111, 1014},
    {400, 111, 1244}
    };
    
    
    // for wireless signal strength
    // {x, y, strength}
    public double[][] ws = {
    	{941.5, 63, 25}, // outside robotics door
    	{1001.5, 63, 27}, 
    	{1061.5, 63, 24},
    	{1121.5, 63, 28},
    	{1181.5, 63, 32},
    	{1241.5, 63, 30},
    	{1301.5, 63, 40}, // end of hall W
    	{881.5, 63, 25},
    	{821.5, 63, 27},
    	{761.5, 63, 36},
    	{701.5, 63, 30},
    	{641.5, 63, 38}, //5
    	{581.5, 63, 82},
    	{521.5, 63, 40},
    	{461.5, 63, 36},
    	{401.5, 63, 32},
    	{341.5, 63, 38}, //10
    	{281.5, 63, 45},
    	{221.5, 63, 36},
    	{161.5, 63, 28},
    	{101.5, 63, 29},
    	{41.5, 63, 25}, // end of hall E
    	{573, 111, 90}, // top of turing hall
    	{573, 171, 82},
    	{573, 231, 90},
    	{573, 291, 38},
    	{573, 351, 32},
    	{573, 411, 28},
    	{573, 471, 30}, // end of turing hall
    	{1295, 111, 40}, // top of beckman hall
    	{1295, 171, 90},
    	{1295, 231, 70},
    	{1295, 291, 90},
    	{1295, 351, 90} // end of beckman hall
    	
    	
    };
   
    
    public EnvMap(int type)
    {
    	this.type = type;
        AT = null;
        obstacles = new Vector();
        wireless = new Vector();
        observed = new Vector();
        
        initialize(type);
    }
    
    public void initialize(int type)
    {
    	switch (type)
    	{
    		case EnvMap.MAP:
    		    /*
    		    this.addObs(new Rectangle2D.Float(-100,-100,200,5));
		        this.addObs(new Rectangle2D.Float(100,-100,5,200));
		        this.addObs(new Rectangle2D.Float(-100,-100,5,200));
		        this.addObs(new Rectangle2D.Float(-100,100,200,5));
		        */
		        //H.pl("hc data: hc.length = " + hc.length + " hc[0].length = " + hc[0].length);
		        //H.pl("vc data: vc.length = " + vc.length + " vc[0].length = " + vc[0].length);
		        
		        for (int i=0 ; i<hc.length ; ++i)
		        {
		        	this.addObs(makeWall(hc[i][0],hc[i][2],hc[i][1],hc[i][2]));
		        }
		        for (int i=0 ; i<vc.length ; ++i)
		        {
		        	this.addObs(makeWall(vc[i][2],vc[i][0],vc[i][2],vc[i][1]));
		        }
		        
		        // add in wireless Signal
		        for (int i=0 ; i<ws.length ; ++i)
		        {
		        	wireless.add(makeWireless(ws[i][0], ws[i][1]));
		        }
		        findBounds();
    		    break;
    		
    		case EnvMap.IR:
    		    // need three triangles and an ER1 homeplate
    		    ir_nw = makeIR(-50,0,-90,15,40);
    		    ir_n = makeIR(-25,0,-90,15,60);
    		    ir_ne = makeIR(25,0,-90,15,20);
    		    wi = makeIR(50,0, -90,15, 0);
    		    break;
    		
    		case EnvMap.BLANK:
    		    // no obstacles added
    		    break;
    		    
    		default: 
    			H.pl("map type not recognized in EnvMap.initialize");
    			break;
    	}
    }
    
    public void findBounds()
    {
    	;// done in makeWall right now 
    	size_x = Math.abs(env_min_x - env_max_x);
    	size_y = Math.abs(env_min_y - env_max_y);
    }
    
    public Ellipse2D.Double makeWireless(double x, double y)
    {
    	// conversion factors for HMC: _X = -576, _Y = -65, SCALE = 2.54
    	double cF = MAP_SCALE;
    	double dX = MAP_OFFSET_X;
    	double dY = MAP_OFFSET_Y;
    	double nx=cF*(x+dX), ny=cF*(y+dY);
    	if (nx > env_max_x)  env_max_x = nx;
    	if (nx < env_min_x)  env_min_x = nx;
    	if (ny > env_max_y)  env_max_y = ny;
    	if (ny < env_min_y)  env_min_y = ny;
    	// ellipse is 15 wide, 15 high
    	return new Ellipse2D.Double(nx,ny,15,15);
    }
    
    public Line2D.Double makeWall(double x, double y, double x2, double y2)
    {
        // conversion factors for HMC: _X = -576, _Y = -65, SCALE = 2.54
    	double cF = MAP_SCALE;
    	double dX = MAP_OFFSET_X;
    	double dY = MAP_OFFSET_Y;
    	double nx=cF*(x+dX), ny=cF*(y+dY), nx2=cF*(x2+dX), ny2=cF*(y2+dY);
    	if (nx > env_max_x)  env_max_x = nx;
    	if (nx < env_min_x)  env_min_x = nx;
    	if (ny > env_max_y)  env_max_y = ny;
    	if (ny < env_min_y)  env_min_y = ny;
    	if (nx2 > env_max_x)  env_max_x = nx2;
    	if (nx2 < env_min_x)  env_min_x = nx2;
    	if (ny2 > env_max_y)  env_max_y = ny2;
    	if (ny2 < env_min_y)  env_min_y = ny2;
    	return new Line2D.Double(nx,ny,nx2,ny2);
    }
    
    // th and alpha in degrees...; d is distance; x,y is the starting point
    public GeneralPath makeIR(double x, double y, double th, double alpha, double d)
    {
    	th = Math.toRadians(th);
    	alpha = Math.toRadians(alpha);
    	// start at (x,y)
    	GeneralPath polygon = new GeneralPath();
        polygon.moveTo((float)x,(float)y);
        polygon.lineTo((float)(x + d*Math.cos(th-alpha)),(float)(y+d*Math.sin(th-alpha)));
        polygon.lineTo((float)(x + d*Math.cos(th+alpha)),(float)(y+d*Math.sin(th+alpha)));
        polygon.closePath();
        
        return polygon;
    }
    
    public void addObs(Shape s)
    {
        obstacles.add(s);   
    }
    
    public void addWall(Line2D.Double w)
    {
    	observed.add(w);
    }
    
    public void draw(Graphics2D g)
    {
        nextStep();
        AffineTransform savedAT = g.getTransform();
        
        //AT = AffineTransform.getScaleInstance(zoom,zoom);
        //AT.rotate(Math.toRadians(theta));
        //AT.translate(-w/2,-h/2);
        //g.transform(AT);
        
        g.setColor(Color.blue);
        int listsize = obstacles.size();
        for (int i=0 ; i<listsize ; ++i)
        {
        	Shape s = (Shape)(obstacles.get(i));
        	if (s instanceof Line2D.Double) // drawing walls
        	{
        		g.draw(s);
        	}
        	else // filling in other shapes
        	{
            	g.fill((Shape)(obstacles.get(i)));
            }
        }
        
        
       	for (int i = 0; i < wireless.size(); ++i) 
       	{
       		setColor(g, ws[i][2]);
       		g.fill((Shape)wireless.get(i));
       	}
        /*
        if (extraSeg != null) // for helping with geometry code
        {
        	g.setColor(Color.red);
        	g.draw(extraSeg);
        }
        */
        g.setColor(Color.black);
        //H.pl("walls: " + observed.size());
        for (int i = 0; i < observed.size(); ++i)
        {
        	g.draw((Line2D.Double)observed.get(i));
        }
        
        
        if (this.type == EnvMap.IR)
        {
        	ir_nw = makeIR(-100,0,-90,15,(float)CS5App.er1.ir_nw);
		    ir_n = makeIR(-33,0,-90,15,(float)CS5App.er1.ir_n);
		    ir_ne = makeIR(33,0,-90,15,(float)CS5App.er1.ir_ne);
		    wi = makeIR(100,0,-90,15,(float)CS5App.er1.wi);
		    //H.pl("wi - " + (float)CS5App.er1.wi);
        	g.setColor(Color.white);
        	g.fill(ir_nw);
        	g.fill(ir_n);
        	g.fill(ir_ne);
        	g.setColor(Color.orange);
        	g.fill(wi);
        }
        g.setTransform(savedAT);
        
        printLines();
    }
    
    public void setColor(Graphics g, double s)
    {
    	if (s < 33) 
    		g.setColor(Color.red);
    	else if (s < 66)
    		g.setColor(Color.orange);
    	else
    		g.setColor(Color.green);
    }
    
    public void printLines()
    {
    	if (true) return;
    	
        for (int i=0 ; i<obstacles.size() ; ++i)
        {
            Shape s = (Shape)(obstacles.get(i));
            PathIterator j = s.getPathIterator(null); // null transform
            while (!j.isDone())
            {
                float[] coords = new float[6];
                int currentSegType = j.currentSegment(coords);
                float moveTo_x = 0.0f;
                float moveTo_y = 0.0f;
                float last_x = 0.0f;
                float last_y = 0.0f;
                
                switch (currentSegType)
                {
                    case PathIterator.SEG_MOVETO:
                    {
                        moveTo_x = coords[0];
                        moveTo_y = coords[1];
                        H.pl("MOVETO segment: " + moveTo_x + "," + moveTo_y);
                        last_x = moveTo_x;
                        last_y = moveTo_y;
                        break;
                    }
                    case PathIterator.SEG_CLOSE:
                    {
                        float x = moveTo_x;
                        float y = moveTo_y;
                        H.pl("CLOSE segment: " + x + "," + y + "  from  " + last_x + "," + last_y);
                        last_x = x;
                        last_y = y;
                        break;
                    }
                    case PathIterator.SEG_LINETO:
                    {
                        float x = coords[0];
                        float y = coords[1];
                        H.pl("LINETO segment: " + x + "," + y + "  from  " + last_x + "," + last_y);
                        last_x = x;
                        last_y = y;
                        break;
                    }
                    case PathIterator.SEG_QUADTO:
                    {
                        H.pl("Quadto segment");
                        break;
                    }
                    case PathIterator.SEG_CUBICTO:
                    {
                        H.pl("Cubicto segment");
                        break;
                    }
                    default:
                    {
                        H.pl("Unhandled segment type...");
                        break;
                    }
                }
                j.next();
            } // end while (!j.isDone())
        }
        
        
    }
    
    public void nextStep()
    {
        ; // degrees
    }
    
    public void handleResizeEvent()
    {
        ;
    }
    
    /* 
     *Return the wireless signal according to the map at location (x,y)
     **/
    public double getWiSig(double x, double y)
    {
    	double temp1 = -1;
    	int index1 = wireless.size();
    	double dist;
    	Ellipse2D.Double p;
    	for (int i = 0; i < wireless.size(); ++i) 
    	{
    		p = (Ellipse2D.Double)wireless.get(i);
    		dist =  (x - p.x) * (x - p.x) +
    				(y - p.y) * (y - p.y);
    		//H.pl(x + " " + y + " " + p.x + " " + p.y);
    		if (temp1 < 0 || dist < temp1)
    		{
    			temp1 = dist;
    			index1 = i;
    		}
    	}
    	//H.pl("found " + index1 + " and " + temp1);
    	
    	return ws[index1][2];
    }
}

















    