import java.util.Vector;
import javax.swing.*;
import H.*;

class MyUpdater extends Thread 
{

    public Vector itemList;    // locations to update
    public boolean waiting = true;
    public int sleepTime = 0;
    public int stepsToRun = 0;  // if this is > 0, the Thread waits
                                // after this many steps
    
    public MyUpdater()
    {
    	
        itemList = new Vector(); 
        sleepTime = 100;
    }
    
    public MyUpdater(int st)
    {
    	
    	itemList = new Vector();
    	sleepTime = st;
    }
    
    public void addItem(Object ob)
    {
        itemList.add(ob);
    }
    
    public void setSleepTime(int sleepTimeIn)
    {
        this.sleepTime = sleepTimeIn;
    }
    
    // alternates between waiting and going
    public synchronized void toggle()
    {
        waiting = !waiting;
        this.notifyAll();
    }
    
    public synchronized void off()
    {
        waiting = true;
        this.notifyAll();
    }
    
    public synchronized void step(int i)
    {
        if (i>0) stepsToRun = i;
        waiting = false;
        this.notifyAll();
    }
    
    public synchronized void on()
    {
        waiting = false;
        this.notifyAll();
    }
    
    // run() is what the thread does
    public void run()
    {
        while (true) 
        {
            try
            {
                // we sleep for a bit, if desired
                if (sleepTime > 0)
                    Thread.sleep(sleepTime);
                
                // we wait using this object as our lock
                // waiting is indefinite sleeping and is
                // woken when notify() or notifyAll() is
                // called by the locking object. 
                // this is done in toggle()
                synchronized(this)
                {
                    while (waiting)
                        this.wait();
                }
            }                               
            catch (InterruptedException e)
            {
                ;
            } 

            // this call updates everything we'd like to draw
            // Drawable.nextStepList();
            //er1.updateInSimulation();
            
            // this will queue a request to draw everything
            // all drawing has to be done in the GUI thread
            // see the URL in MyPanel for details
            for (int i=0 ; i<itemList.size() ; ++i)
            {
                ((Updatable)itemList.get(i)).threadUpdate(); // repaint
            }
            
            // check if there are a finite number of steps to run
            if (stepsToRun > 0)
            {
                --stepsToRun;
                
                // now, if it's zero, we stop
                if (stepsToRun == 0)
                {
                    waiting = true;
                }
            }
                
        }
    } // end run method
    
}
	